'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createBidirectional = exports.create = exports.batchCreateBidirectional = exports.batchCreate = exports.find = exports.clearState = exports.SUBJECT_RELATIONS_UPDATED = exports.RELATION_SAVE_REJECTED = exports.RELATION_SAVE_FULFILLED = exports.RELATION_SAVE_STARTED = exports.RELATION_FIND_REJECTED = exports.RELATION_FIND_FULFILLED = exports.RELATION_FIND_STARTED = exports.CLEAR_RELATION_STATE = undefined;

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _cspace = require('./cspace');

var _cspace2 = _interopRequireDefault(_cspace);

var _reducers = require('../reducers');

var _errorCodes = require('../constants/errorCodes');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var CLEAR_RELATION_STATE = exports.CLEAR_RELATION_STATE = 'CLEAR_RELATION_STATE';
var RELATION_FIND_STARTED = exports.RELATION_FIND_STARTED = 'RELATION_FIND_STARTED';
var RELATION_FIND_FULFILLED = exports.RELATION_FIND_FULFILLED = 'RELATION_FIND_FULFILLED';
var RELATION_FIND_REJECTED = exports.RELATION_FIND_REJECTED = 'RELATION_FIND_REJECTED';
var RELATION_SAVE_STARTED = exports.RELATION_SAVE_STARTED = 'RELATION_SAVE_STARTED';
var RELATION_SAVE_FULFILLED = exports.RELATION_SAVE_FULFILLED = 'RELATION_SAVE_FULFILLED';
var RELATION_SAVE_REJECTED = exports.RELATION_SAVE_REJECTED = 'RELATION_SAVE_REJECTED';
var SUBJECT_RELATIONS_UPDATED = exports.SUBJECT_RELATIONS_UPDATED = 'SUBJECT_RELATIONS_UPDATED';

var clearState = exports.clearState = function clearState() {
  return {
    type: CLEAR_RELATION_STATE
  };
};

/*
 * Find a relation, given at least the subject csid and object csid, and optionally the subject
 * record type, object record type, and predicate. This function assumes that there will be very
 * few (typically zero or one) results, and retrieves them all without paginating.
 */
var find = exports.find = function find(config, descriptor) {
  return function (dispatch, getState) {
    var subject = descriptor.subject,
        object = descriptor.object,
        predicate = descriptor.predicate;


    if (!subject.csid || !object.csid) {
      throw new Error('subject csid and object csid must be supplied');
    }

    if ((0, _reducers.getRelationFindResult)(getState(), descriptor)) {
      // Already have a result for this descriptor. Do nothing.
      // TODO: Also check for a pending find request.

      return null;
    }

    dispatch({
      type: RELATION_FIND_STARTED,
      meta: {
        subject: subject,
        object: object,
        predicate: predicate
      }
    });

    var params = {
      prd: predicate,
      wf_deleted: 'false',
      pgSz: '0'
    };

    if (subject) {
      var csid = subject.csid,
          recordType = subject.recordType;


      params.sbj = csid;

      if (recordType) {
        params.sbjType = (0, _get2.default)(config, ['recordTypes', recordType, 'serviceConfig', 'objectName']);
      }
    }

    if (object) {
      var _csid = object.csid,
          _recordType = object.recordType;


      params.obj = _csid;

      if (_recordType) {
        params.objType = (0, _get2.default)(config, ['recordTypes', _recordType, 'serviceConfig', 'objectName']);
      }
    }

    var requestConfig = {
      params: params
    };

    return (0, _cspace2.default)().read('/relations', requestConfig).then(function (response) {
      return dispatch({
        type: RELATION_FIND_FULFILLED,
        payload: response,
        meta: {
          subject: subject,
          object: object,
          predicate: predicate
        }
      });
    }, function (error) {
      return dispatch({
        type: RELATION_FIND_REJECTED,
        payload: {
          code: _errorCodes.ERR_API,
          error: error
        },
        meta: {
          subject: subject,
          object: object,
          predicate: predicate
        }
      });
    });
  };
};

var doCreate = function doCreate(subject, object, predicate) {
  return function (dispatch) {
    dispatch({
      type: RELATION_SAVE_STARTED,
      meta: {
        subject: subject,
        object: object,
        predicate: predicate
      }
    });

    var config = {
      data: {
        document: {
          '@name': 'relations',
          'ns2:relations_common': {
            '@xmlns:ns2': 'http://collectionspace.org/services/relation',
            subjectCsid: subject.csid,
            objectCsid: object.csid,
            relationshipType: predicate
          }
        }
      }
    };

    return (0, _cspace2.default)().create('/relations', config).then(function (response) {
      return dispatch({
        type: RELATION_SAVE_FULFILLED,
        payload: response,
        meta: {
          subject: subject,
          object: object,
          predicate: predicate
        }
      });
    }).catch(function (error) {
      dispatch({
        type: RELATION_SAVE_REJECTED,
        payload: {
          code: _errorCodes.ERR_API,
          error: error
        },
        meta: {
          subject: subject,
          object: object,
          predicate: predicate
        }
      });

      return Promise.reject(error);
    });
  };
};

var batchCreate = exports.batchCreate = function batchCreate(subject, objects, predicate) {
  return function (dispatch) {
    return Promise.all(objects.map(function (object) {
      return dispatch(doCreate(subject, object, predicate));
    })).then(function () {
      return dispatch({
        type: SUBJECT_RELATIONS_UPDATED,
        meta: subject
      });
    });
  };
};

var batchCreateBidirectional = exports.batchCreateBidirectional = function batchCreateBidirectional(subject, objects, predicate) {
  return function (dispatch) {
    return Promise.all(objects.map(function (object) {
      return dispatch(doCreate(subject, object, predicate)).then(function () {
        return dispatch(doCreate(object, subject, predicate));
      });
    })).then(function () {
      return dispatch({
        type: SUBJECT_RELATIONS_UPDATED,
        meta: subject
      });
    }).catch(function () {});
  };
};

var create = exports.create = function create(subject, object, predicate) {
  return function (dispatch) {
    return dispatch(doCreate(subject, object, predicate)).then(function () {
      return dispatch({
        type: SUBJECT_RELATIONS_UPDATED,
        meta: subject
      });
    }).catch(function () {});
  };
};

var createBidirectional = exports.createBidirectional = function createBidirectional(subject, object, predicate) {
  return function (dispatch) {
    return dispatch(doCreate(subject, object, predicate)).then(function () {
      return dispatch(doCreate(object, subject, predicate));
    }).then(function () {
      return dispatch({
        type: SUBJECT_RELATIONS_UPDATED,
        meta: subject
      });
    }).catch(function () {});
  };
};